/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package components

import (
	instance "k8s.libre.sh/instance"
	meta "k8s.libre.sh/meta"
	deployment "k8s.libre.sh/objects/deployment"
	parameters "k8s.libre.sh/settings"
)

// +kubebuilder:object:generate=true
type Workload struct {
	*meta.ObjectMeta `json:"commonMeta,omitempty"`
	//*Settings              `json:"settings,omitempty"`
	*deployment.Deployment `json:"deployment,omitempty"`
	*Network               `json:"network,omitempty"`
	Dependencies           bool `json:"dependencies,omitempty"`
}

func (w *Workload) HasDependencies() bool {
	return w.Dependencies
}

func (w *Workload) GetMeta() instance.Meta {
	return w.ObjectMeta
}

func (w *Workload) GetObjects() map[int]instance.Object {

	objects := make(map[int]instance.Object)

	objects[0] = w.Network.GetService()
	objects[1] = w.Network.GetIngress()
	objects[2] = w.Deployment

	return objects
}

func (w *Workload) SetDependencies(s *parameters.Settings) { w.Settings = *s }
func (w *Workload) GetDependencies() parameters.Interface  { return &w.Settings }

// TODO make a generic function with component interface
// getObjects loop setObjectMetaFromComponent
// merge labels && annotations && finalizers not set
func (w *Workload) SetDefaults(i instance.Instance) {

	// TODO FIXME
	w.Dependencies = true

	// TODO improve initialisation of objectMeta, labels...
	if w.ObjectMeta == nil {
		w.ObjectMeta = new(meta.ObjectMeta)
	}

	if len(w.ObjectMeta.Labels) == 0 {
		w.ObjectMeta.Labels = make(map[string]string)
	}

	// TODO TOFIX
	w.ObjectMeta.SetComponent("app")

	if w.Deployment.ObjectMeta == nil {
		w.Deployment.ObjectMeta = new(meta.ObjectMeta)
	}

	if w.Network.ObjectMeta == nil {
		w.Network.ObjectMeta = new(meta.ObjectMeta)
	}

	if w.Network.Ingress.ObjectMeta == nil {
		w.Network.Ingress.ObjectMeta = new(meta.ObjectMeta)
	}

	if w.Network.Service.ObjectMeta == nil {
		w.Network.Service.ObjectMeta = new(meta.ObjectMeta)
	}

	instance.InitComponent(i, w)
}
