package instance

import (
	"github.com/presslabs/controller-util/syncer"
	interfaces "k8s.libre.sh/interfaces"
	parameters "k8s.libre.sh/settings"
)

type Component interface {
	Meta
	GetMeta() Meta
	SetDefaults(i Instance)
	GetObjects() map[int]Object
	HasDependencies() bool
	GetDependencies() parameters.Interface
	//	SetDependencies(*parameters.Settings)
}

func NewSyncer(i Component, r interfaces.Reconcile, owner interfaces.Object) (syncers []syncer.Interface) {
	mutators := i.GetObjects()

	for _, m := range mutators {
		syncers = append(syncers, NewObjectSyncer(m, owner, r))
	}

	return syncers
}

func InitComponent(i Instance, c Component) {

	component := c.GetComponent()

	SetObjectMeta(i, c)

	if len(component) > 0 {
		c.SetComponent(component)
	}

	for _, o := range c.GetObjects() {
		SetObjectMeta(c, o)
	}

	if c.HasDependencies() {
		dependencies := i.GetSettings().GetParameters()

		settings := c.GetDependencies()

		settings = parameters.MergeSettings(dependencies, settings)

	}
}

func MergeSettings(dependencies, settings *parameters.Settings) *parameters.Settings {
	params := parameters.Parameters{}

	if settings == nil {
		settings = dependencies
	} else {

		settings.SetConfigRefs(append(settings.GetConfigRefs(), dependencies.GetConfigRefs()...))
		settings.SetSecretRefs(append(settings.GetSecretRefs(), dependencies.GetSecretRefs()...))

		for _, p := range *dependencies.GetParameters() {
			params = append(params, p)
			settings.Parameters = &params
		}
	}

	return settings
}
