/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package deployment

import (
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
	pod "k8s.libre.sh/objects/pod"

	appsv1 "k8s.io/api/apps/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// +kubebuilder:object:generate=true
type Deployment struct {
	*meta.ObjectMeta `json:"meta,omitempty"`
	*DeploymentSpec  `json:",inline"`
}

// +kubebuilder:object:generate=true
type DeploymentSpec struct {
	Replicas     *int32                    `json:"replicas,omitempty"`
	Strategy     appsv1.DeploymentStrategy `json:"strategy,omitempty"`
	*pod.PodSpec `json:",inline"`
}

func (r *DeploymentSpec) GetReplicas() *int32 {
	return r.Replicas
}

func (r *DeploymentSpec) GetDeploymentStrategy() appsv1.DeploymentStrategy {
	return r.Strategy
}

func (o *Deployment) GetObject() interfaces.Object {
	return &appsv1.Deployment{}
}

func (w *Deployment) Mutate(obj interfaces.Object) error {

	deploy := obj.(*appsv1.Deployment)

	w.PodSpec.ObjectMeta = w.ObjectMeta

	MutateDeployment(w, deploy)

	meta.MutateMeta(w.ObjectMeta, obj)

	deploy.Spec.Template.ObjectMeta.Name = deploy.ObjectMeta.Name

	deploy.Spec.Template.ObjectMeta.Labels = w.ObjectMeta.Labels

	deploy.Spec.Selector = metav1.SetAsLabelSelector(w.ObjectMeta.Labels)

	return nil
}

//func NewDeployment(spec *DeploymentSpec, p parameters.Parameters) *Deployment {
//	return &Deployment{
//		Parameters:     p,
//		DeploymentSpec: spec,
//	}
//}
