/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package settings

const (
	GenerateRand     Generate = "rand"
	GenerateRand12   Generate = "rand12"
	GenerateRand24   Generate = "rand24"
	GenerateTemplate Generate = "template"
)

const (
	MountLiteral MountType = "literal"
	MountFrom    MountType = "from"
	MountFile    MountType = "file"

//	MountEnvVar     MountType = "envVar"
//	MountEnvVarFrom MountType = "envVarFrom"
)

const (
	ConfigParameter      ParameterType = "configmap"
	SecretParameter      ParameterType = "secret"
	ObjectFieldParameter ParameterType = "field"
)

type MountType string
type Generate string
type ParameterType string

// +kubebuilder:object:generate=true
type Parameters []*Parameter

// +kubebuilder:object:generate=true
type Parameter struct {
	// Key that will be mounted as environment variable
	Key string `json:"key,omitempty"`
	// Value of the paramater, should not contain secret values
	Value string `json:"value,omitempty"`
	// ValueFrom specifies where to get the value from for the parameter
	ValueFrom `json:",inline"`
	// MountFrom specifies where the parameter will be mounted as file
	MountPath MountPath `json:"mountFrom,omitempty"`
	// Generate is specified if a secret or configmap data needs to be randomly generated
	Generate Generate `json:"generate,omitempty"`
	// Type specifies if its a ConfigMap or Secret
	Type      ParameterType `json:"type,omitempty"`
	MountType MountType     `json:"mountType,omitempty"`
}

type ValueFrom struct {
	// Key to select from in ConfigMap or Secret
	FromKey string `json:"fromKey,omitempty"`
	// Ref is a reference to a local object, set ParameterType to define the object
	Ref string `json:"ref,omitempty"`
}

type MountPath struct {
	Path    string `json:"path,omitempty"`
	SubPath string `json:"subPath,omitempty"`
}

// +kubebuilder:object:generate=true
type SecretParam struct {
	// Key that will be mounted as environment variable
	Key string `json:"key,omitempty"`
	// ValueFrom specifies where to get the value from for the parameter
	ValueFrom `json:",inline"`
	// Source of the Secret or ConfigMap
	Destination string `json:"destination,omitempty"`
	// Source of the Secret or ConfigMap
	Generate bool `json:"generate,omitempty"`
}
