/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package settings

import (
	corev1 "k8s.io/api/core/v1"
)

type Settings struct {
	*EnvFrom    `json:",inline"`
	*Parameters `json:"parameters,omitempty"`
}

// +kubebuilder:object:generate=true
type EnvFrom struct {
	SecretRefs []string `json:"secretRefs,omitempty"`
	ConfigRefs []string `json:"configRefs,omitempty"`
}

func (s *Settings) SetParameters(parameters *Parameters) {
	s.Parameters = parameters
}

func (s *Settings) GetParameters() *Parameters {
	if s.Parameters == nil {
		s.Parameters = new(Parameters)
	}
	return s.Parameters
}

func (s *Settings) GetSecretRefs() []string {
	if s.EnvFrom != nil {
		return s.EnvFrom.SecretRefs
	}

	return nil
}

func (s *Settings) SetSecretRefs(refs []string) {
	if s.EnvFrom == nil {
		s.EnvFrom = new(EnvFrom)
	}
	s.SecretRefs = refs
}

func (s *Settings) GetConfigRefs() []string {
	if s.EnvFrom != nil {
		return s.EnvFrom.ConfigRefs
	}

	return nil
}

func (s *Settings) SetConfigRefs(refs []string) {
	if s.EnvFrom == nil {
		s.EnvFrom = new(EnvFrom)
	}
	s.ConfigRefs = refs
}

/*
type SecretParameters struct {
	Refs        []string `json:"configRefs,omitempty"`
	*Parameters `json:"parameters,omitempty"`
}

func (s *Settings) GetEnvFrom() []corev1.EnvFromSource {
	envFrom := []corev1.EnvFromSource{}
	if len(s.Secrets.GetEnvFrom()) > 0 {
		envFrom = append(envFrom, s.Secrets.GetEnvFrom()...)
	}
	if len(s.Parameters.GetEnvFrom()) > 0 {
		envFrom = append(envFrom, s.Parameters.GetEnvFrom()...)
	}
	return envFrom
}

func (e *SecretParameters) GetEnvFrom() []corev1.EnvFromSource {
	envFroms := []corev1.EnvFromSource{}
	envFrom := corev1.EnvFromSource{}

	if e != nil {
		//		if len(e.Refs) > 0 {
		for _, ref := range e.Refs {
			envFrom = corev1.EnvFromSource{
				SecretRef: &corev1.SecretEnvSource{
					LocalObjectReference: corev1.LocalObjectReference{
						Name: ref,
					},
				},
			}
			envFroms = append(envFroms, envFrom)
		}
		//		}
	}

	if len(envFroms) > 0 {
		return envFroms
	}

	return nil
}

type ConfigParameters struct {
	Refs        []string `json:"configRefs,omitempty"`
	*Parameters `json:"parameters,omitempty"`
}

func (e *ConfigParameters) GetEnvFrom() []corev1.EnvFromSource {
	envFroms := []corev1.EnvFromSource{}
	envFrom := corev1.EnvFromSource{}

	if e != nil {
		//		if len(e.Refs) > 0 {
		for _, ref := range e.Refs {
			envFrom = corev1.EnvFromSource{
				SecretRef: &corev1.SecretEnvSource{
					LocalObjectReference: corev1.LocalObjectReference{
						Name: ref,
					},
				},
			}
			envFroms = append(envFroms, envFrom)
		}
		//		}
	}

	if len(envFroms) > 0 {
		return envFroms
	}

	return nil
}
*/
func (e *EnvFrom) GetEnvFrom() []corev1.EnvFromSource {
	envFroms := []corev1.EnvFromSource{}
	envFrom := corev1.EnvFromSource{}

	if e != nil {
		if len(e.ConfigRefs) > 0 {
			for _, configRef := range e.ConfigRefs {
				envFrom = corev1.EnvFromSource{
					ConfigMapRef: &corev1.ConfigMapEnvSource{
						LocalObjectReference: corev1.LocalObjectReference{
							Name: configRef,
						},
					},
				}
				envFroms = append(envFroms, envFrom)
			}
		}

		if len(e.SecretRefs) > 0 {
			for _, secretRef := range e.SecretRefs {
				envFrom = corev1.EnvFromSource{
					SecretRef: &corev1.SecretEnvSource{
						LocalObjectReference: corev1.LocalObjectReference{
							Name: secretRef,
						},
					},
				}
				envFroms = append(envFroms, envFrom)
			}
		}
	}

	if len(envFroms) > 0 {
		return envFroms
	}

	return nil
}

// We need to manually change this autogenerated function as it is broken with Paramete drs type, its trying to do new([]*Parameter) instead of new(Parameters)
// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *Settings) DeepCopyInto(out *Settings) {
	*out = *in
	if in.EnvFrom != nil {
		in, out := &in.EnvFrom, &out.EnvFrom
		*out = new(EnvFrom)
		(*in).DeepCopyInto(*out)
	}
	if in.Parameters != nil {
		in, out := &in.Parameters, &out.Parameters
		*out = new(Parameters)
		(*in).DeepCopyInto(*out)
	}
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Settings.
func (in *Settings) DeepCopy() *Settings {
	if in == nil {
		return nil
	}
	out := new(Settings)
	in.DeepCopyInto(out)
	return out
}
